#include "mian.h"

#define MAX_COUNT 50 // Max number of keystrokes to remember

#pragma data_seg(".shared")  // ".shared" is defined in exports.def to allow
                             //  all instances of the dll to share these variables
HWND	m_hHwndKey = 0;
HHOOK	m_hHookKey = 0;           
HWND	m_hHwndMouse = 0;
HHOOK	m_hHookMouse = 0;

CHAR	m_szMain[MAX_COUNT] = ""; // A buffer to store decoded keyboard codes
int		m_nLastPos = 0;           // Last position used in the buffer

HCURSOR m_hCursorLast = 0;        // These two values are used to copy the mouse
HCURSOR m_hCursorLastRet = 0;     //  cursor
#pragma data_seg()

// Set the values for the window and hook for the Keyboard hook
void WINAPI SetValuesKey(HWND hWnd, HHOOK hk) {
	m_hHwndKey = hWnd;
	m_hHookKey = hk;
}

// Set the values for the window and hook for the mouse hook
void WINAPI SetValuesMouse(HWND hWnd, HHOOK hk) {
	m_hHwndMouse = hWnd;
	m_hHookMouse = hk;
}

// This is the mouse hook itself
LRESULT CALLBACK MouseProc( int nCode, WPARAM wParam, LPARAM lParam ) {
	if (nCode < 0) {
		return CallNextHookEx(m_hHookMouse, nCode, wParam, lParam);
	}

	if (nCode == HC_ACTION) {

		HCURSOR hCursor = GetCursor();

		// Copy the mouse cursor
		if (hCursor != m_hCursorLastRet) {
			if (m_hCursorLast) {
				DestroyCursor(m_hCursorLast);
			}

			ICONINFO ii;
			GetIconInfo(hCursor, &ii);
			m_hCursorLast = CreateIconIndirect(&ii);
			DeleteObject(ii.hbmMask);
			DeleteObject(ii.hbmColor);
;
			m_hCursorLastRet = hCursor;

		}

		// If the message is a WM_NC*, convert it to a WM_*
		if ((wParam >= 0xA0) & (wParam <= 0xA9)) {
			wParam = wParam + 352;
		}

		// Let the listening window know about the message
		PostMessage(m_hHwndMouse, wParam, 0, MAKEWPARAM(
			( (MOUSEHOOKSTRUCT*) lParam )->pt.x,
			( (MOUSEHOOKSTRUCT*) lParam )->pt.y));

	}


	return 0;
}

// Function to get the last mouse cursor that was copied.  The return cursor
//  is a copy of the copy, so the callee must delete it.
HCURSOR WINAPI GetLastCursor() {
	HCURSOR hCursor = m_hCursorLast;
	m_hCursorLast = 0;
	return (hCursor);
}

// Returns the current state of the keyboard bufffer
void WINAPI GetKeyInfo(CHAR * szIn, int * nLastPos) {
	memcpy(szIn, &m_szMain, MAX_COUNT);
	(* nLastPos) = m_nLastPos;
}

// Keyboard hook
LRESULT CALLBACK KeyProc( int nCode, WPARAM wParam, LPARAM lParam ) {
	if (nCode < 0) {
		return CallNextHookEx(m_hHookKey, nCode, wParam, lParam);
	}

	if (nCode == HC_ACTION) {

		if (lParam & 0x80000000) {
			PostMessage(m_hHwndKey, WM_KEYUP, wParam, lParam);
		} else {
			BYTE lpKeyState[256];
			GetKeyboardState((unsigned char *)&lpKeyState);

			// Convert the message to a normal keystroke, and store it
			//  in the buffer
			WORD lpChar;

			int nRet = ToAscii( 
				wParam, 
				(lParam & 0xFF0000) >> 16, 
				(unsigned char *)&lpKeyState, 
				&lpChar, 
				0); 
 
			if (nRet == 1) {
				m_szMain[m_nLastPos] = (char) lpChar;
				m_nLastPos ++;
				if (m_nLastPos >= MAX_COUNT) {
					m_nLastPos = 0;
				}
			}

			PostMessage(m_hHwndKey, WM_KEYDOWN, wParam, lParam);
		}

	}

	return 0;
}

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID) {
	if (fdwReason == DLL_PROCESS_ATTACH) {
		DisableThreadLibraryCalls(hinstDLL);
	}
	return TRUE;
}

// This is to prevent the CRT from loading, thus making this a smaller
//  and faster dll.
extern "C" BOOL __stdcall _DllMainCRTStartup( HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {
    return DllMain( hinstDLL, fdwReason, lpvReserved );
}
